=begin
#Lob

#The Lob API is organized around REST. Our API is designed to have predictable, resource-oriented URLs and uses HTTP response codes to indicate any API errors. <p> Looking for our [previous documentation](https://lob.github.io/legacy-docs/)? 

The version of the OpenAPI document: 1.3.0
Contact: lob-openapi@lob.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 5.2.1

=end

require 'cgi'

module Lob
  class CardOrdersApi
    attr_accessor :api_client

    def initialize(api_client = ApiClient.default)
      @api_client = api_client
    end
    # create
    # Creates a new card order given information
    # @param card_id [String] The ID of the card to which the card orders belong.
    # @param card_order_editable [CardOrderEditable] 
    # @param [Hash] opts the optional parameters
    # @return [CardOrder]
    def create(card_id, card_order_editable, opts = {})
      data, _status_code, _headers = card_order_create_with_http_info(card_id, card_order_editable, opts)
      data
    end

    # create
    # Creates a new card order given information
    # @param card_id [String] The ID of the card to which the card orders belong.
    # @param card_order_editable [CardOrderEditable] 
    # @param [Hash] opts the optional parameters
    # @return [Array<(CardOrder, Integer, Hash)>] CardOrder data, response status code and response headers
    def card_order_create_with_http_info(card_id, card_order_editable, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: CardOrdersApi.card_order_create ...'
      end
      # verify the required parameter 'card_id' is set
      if @api_client.config.client_side_validation && card_id.nil?
        fail ArgumentError, "Missing the required parameter 'card_id' when calling CardOrdersApi.card_order_create"
      end
      pattern = Regexp.new(/^card_[a-zA-Z0-9]+$/)
      if @api_client.config.client_side_validation && card_id !~ pattern
        fail ArgumentError, "invalid value for 'card_id' when calling CardOrdersApi.card_order_create, must conform to the pattern #{pattern}."
      end

      # verify the required parameter 'card_order_editable' is set
      if @api_client.config.client_side_validation && card_order_editable.nil?
        fail ArgumentError, "Missing the required parameter 'card_order_editable' when calling CardOrdersApi.card_order_create"
      end
      # resource path
      local_var_path = '/cards/{card_id}/orders'.sub('{' + 'card_id' + '}', CGI.escape(card_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])
      # HTTP header 'Content-Type'
      content_type = @api_client.select_header_content_type(['application/json', 'application/x-www-form-urlencoded', 'multipart/form-data'])
      if !content_type.nil?
          header_params['Content-Type'] = content_type
      end

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:debug_body] || @api_client.object_to_http_body(card_order_editable)

      # return_type
      return_type = opts[:debug_return_type] || 'CardOrder'

      # auth_names
      auth_names = opts[:debug_auth_names] || ['basicAuth']

      new_options = opts.merge(
        :operation => :"CardOrdersApi.card_order_create",
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: CardOrdersApi#card_order_create\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # get
    # Retrieves the card orders associated with the given card id.
    # @param card_id [String] The ID of the card to which the card orders belong.
    # @param [Hash] opts the optional parameters
    # @option opts [Integer] :limit How many results to return. (default to 10)
    # @option opts [Integer] :offset An integer that designates the offset at which to begin returning results. Defaults to 0. (default to 0)
    # @return [CardOrderList]
    def get(card_id, opts = {})
      data, _status_code, _headers = card_orders_retrieve_with_http_info(card_id, opts)
      data
    end

    # get
    # Retrieves the card orders associated with the given card id.
    # @param card_id [String] The ID of the card to which the card orders belong.
    # @param [Hash] opts the optional parameters
    # @option opts [Integer] :limit How many results to return. (default to 10)
    # @option opts [Integer] :offset An integer that designates the offset at which to begin returning results. Defaults to 0. (default to 0)
    # @return [Array<(CardOrderList, Integer, Hash)>] CardOrderList data, response status code and response headers
    def card_orders_retrieve_with_http_info(card_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: CardOrdersApi.card_orders_retrieve ...'
      end
      # verify the required parameter 'card_id' is set
      if @api_client.config.client_side_validation && card_id.nil?
        fail ArgumentError, "Missing the required parameter 'card_id' when calling CardOrdersApi.card_orders_retrieve"
      end
      pattern = Regexp.new(/^card_[a-zA-Z0-9]+$/)
      if @api_client.config.client_side_validation && card_id !~ pattern
        fail ArgumentError, "invalid value for 'card_id' when calling CardOrdersApi.card_orders_retrieve, must conform to the pattern #{pattern}."
      end

      if @api_client.config.client_side_validation && !opts[:'limit'].nil? && opts[:'limit'] > 100
        fail ArgumentError, 'invalid value for "opts[:"limit"]" when calling CardOrdersApi.card_orders_retrieve, must be smaller than or equal to 100.'
      end

      if @api_client.config.client_side_validation && !opts[:'limit'].nil? && opts[:'limit'] < 1
        fail ArgumentError, 'invalid value for "opts[:"limit"]" when calling CardOrdersApi.card_orders_retrieve, must be greater than or equal to 1.'
      end

      # resource path
      local_var_path = '/cards/{card_id}/orders'.sub('{' + 'card_id' + '}', CGI.escape(card_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'limit'] = opts[:'limit'] if !opts[:'limit'].nil?
      query_params[:'offset'] = opts[:'offset'] if !opts[:'offset'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:debug_body]

      # return_type
      return_type = opts[:debug_return_type] || 'CardOrderList'

      # auth_names
      auth_names = opts[:debug_auth_names] || ['basicAuth']

      new_options = opts.merge(
        :operation => :"CardOrdersApi.card_orders_retrieve",
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: CardOrdersApi#card_orders_retrieve\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end
  end
end
