=begin
#Lob

#The Lob API is organized around REST. Our API is designed to have predictable, resource-oriented URLs and uses HTTP response codes to indicate any API errors. <p> Looking for our [previous documentation](https://lob.github.io/legacy-docs/)? 

The version of the OpenAPI document: 1.3.0
Contact: lob-openapi@lob.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 5.2.1

=end

require 'cgi'

module Lob
  class SelfMailersApi
    attr_accessor :api_client

    def initialize(api_client = ApiClient.default)
      @api_client = api_client
    end
    # create
    # Creates a new self_mailer given information
    # @param self_mailer_editable [SelfMailerEditable] 
    # @param [Hash] opts the optional parameters
    # @option opts [String] :idempotency_key A string of no longer than 256 characters that uniquely identifies this resource. For more help integrating idempotency keys, refer to our [implementation guide](https://www.lob.com/guides#idempotent_request). 
    # @return [SelfMailer]
    def create(self_mailer_editable, opts = {})
      data, _status_code, _headers = self_mailer_create_with_http_info(self_mailer_editable, opts)
      data
    end

    # create
    # Creates a new self_mailer given information
    # @param self_mailer_editable [SelfMailerEditable] 
    # @param [Hash] opts the optional parameters
    # @option opts [String] :idempotency_key A string of no longer than 256 characters that uniquely identifies this resource. For more help integrating idempotency keys, refer to our [implementation guide](https://www.lob.com/guides#idempotent_request). 
    # @return [Array<(SelfMailer, Integer, Hash)>] SelfMailer data, response status code and response headers
    def self_mailer_create_with_http_info(self_mailer_editable, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: SelfMailersApi.self_mailer_create ...'
      end
      # verify the required parameter 'self_mailer_editable' is set
      if @api_client.config.client_side_validation && self_mailer_editable.nil?
        fail ArgumentError, "Missing the required parameter 'self_mailer_editable' when calling SelfMailersApi.self_mailer_create"
      end
      if @api_client.config.client_side_validation && !opts[:'idempotency_key'].nil? && opts[:'idempotency_key'].to_s.length > 256
        fail ArgumentError, 'invalid value for "opts[:"idempotency_key"]" when calling SelfMailersApi.self_mailer_create, the character length must be smaller than or equal to 256.'
      end

      # resource path
      local_var_path = '/self_mailers'

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])
      # HTTP header 'Content-Type'
      content_type = @api_client.select_header_content_type(['application/json', 'application/x-www-form-urlencoded', 'multipart/form-data'])
      if !content_type.nil?
          header_params['Content-Type'] = content_type
      end
      header_params[:'Idempotency-Key'] = opts[:'idempotency_key'] if !opts[:'idempotency_key'].nil?

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:debug_body] || @api_client.object_to_http_body(self_mailer_editable)

      # return_type
      return_type = opts[:debug_return_type] || 'SelfMailer'

      # auth_names
      auth_names = opts[:debug_auth_names] || ['basicAuth']

      new_options = opts.merge(
        :operation => :"SelfMailersApi.self_mailer_create",
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:POST, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: SelfMailersApi#self_mailer_create\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # delete
    # Completely removes a self mailer from production. This can only be done if the self mailer's `send_date` has not yet passed.
    # @param sfm_id [String] id of the self_mailer
    # @param [Hash] opts the optional parameters
    # @return [SelfMailerDeletion]
    def delete(sfm_id, opts = {})
      data, _status_code, _headers = self_mailer_delete_with_http_info(sfm_id, opts)
      data
    end

    # delete
    # Completely removes a self mailer from production. This can only be done if the self mailer&#39;s &#x60;send_date&#x60; has not yet passed.
    # @param sfm_id [String] id of the self_mailer
    # @param [Hash] opts the optional parameters
    # @return [Array<(SelfMailerDeletion, Integer, Hash)>] SelfMailerDeletion data, response status code and response headers
    def self_mailer_delete_with_http_info(sfm_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: SelfMailersApi.self_mailer_delete ...'
      end
      # verify the required parameter 'sfm_id' is set
      if @api_client.config.client_side_validation && sfm_id.nil?
        fail ArgumentError, "Missing the required parameter 'sfm_id' when calling SelfMailersApi.self_mailer_delete"
      end
      pattern = Regexp.new(/^sfm_[a-zA-Z0-9]+$/)
      if @api_client.config.client_side_validation && sfm_id !~ pattern
        fail ArgumentError, "invalid value for 'sfm_id' when calling SelfMailersApi.self_mailer_delete, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/self_mailers/{sfm_id}'.sub('{' + 'sfm_id' + '}', CGI.escape(sfm_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:debug_body]

      # return_type
      return_type = opts[:debug_return_type] || 'SelfMailerDeletion'

      # auth_names
      auth_names = opts[:debug_auth_names] || ['basicAuth']

      new_options = opts.merge(
        :operation => :"SelfMailersApi.self_mailer_delete",
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:DELETE, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: SelfMailersApi#self_mailer_delete\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # get
    # Retrieves the details of an existing self_mailer. You need only supply the unique self_mailer identifier that was returned upon self_mailer creation.
    # @param sfm_id [String] id of the self_mailer
    # @param [Hash] opts the optional parameters
    # @return [SelfMailer]
    def get(sfm_id, opts = {})
      data, _status_code, _headers = self_mailer_retrieve_with_http_info(sfm_id, opts)
      data
    end

    # get
    # Retrieves the details of an existing self_mailer. You need only supply the unique self_mailer identifier that was returned upon self_mailer creation.
    # @param sfm_id [String] id of the self_mailer
    # @param [Hash] opts the optional parameters
    # @return [Array<(SelfMailer, Integer, Hash)>] SelfMailer data, response status code and response headers
    def self_mailer_retrieve_with_http_info(sfm_id, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: SelfMailersApi.self_mailer_retrieve ...'
      end
      # verify the required parameter 'sfm_id' is set
      if @api_client.config.client_side_validation && sfm_id.nil?
        fail ArgumentError, "Missing the required parameter 'sfm_id' when calling SelfMailersApi.self_mailer_retrieve"
      end
      pattern = Regexp.new(/^sfm_[a-zA-Z0-9]+$/)
      if @api_client.config.client_side_validation && sfm_id !~ pattern
        fail ArgumentError, "invalid value for 'sfm_id' when calling SelfMailersApi.self_mailer_retrieve, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/self_mailers/{sfm_id}'.sub('{' + 'sfm_id' + '}', CGI.escape(sfm_id.to_s))

      # query parameters
      query_params = opts[:query_params] || {}

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:debug_body]

      # return_type
      return_type = opts[:debug_return_type] || 'SelfMailer'

      # auth_names
      auth_names = opts[:debug_auth_names] || ['basicAuth']

      new_options = opts.merge(
        :operation => :"SelfMailersApi.self_mailer_retrieve",
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: SelfMailersApi#self_mailer_retrieve\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end

    # list
    # Returns a list of your self_mailers. The self_mailers are returned sorted by creation date, with the most recently created self_mailers appearing first.
    # @param [Hash] opts the optional parameters
    # @option opts [Integer] :limit How many results to return. (default to 10)
    # @option opts [String] :before A reference to a list entry used for paginating to the previous set of entries. This field is pre-populated in the &#x60;previous_url&#x60; field in the return response. 
    # @option opts [String] :after A reference to a list entry used for paginating to the next set of entries. This field is pre-populated in the &#x60;next_url&#x60; field in the return response. 
    # @option opts [Array<String>] :include Request that the response include the total count by specifying &#x60;include[]&#x3D;total_count&#x60;. 
    # @option opts [Hash<String, Time>] :date_created Filter by date created.
    # @option opts [Hash<String, String>] :metadata Filter by metadata key-value pair&#x60;.
    # @option opts [Array<SelfMailerSize>] :size The self mailer sizes to be returned.
    # @option opts [Boolean] :scheduled * &#x60;true&#x60; - only return orders (past or future) where &#x60;send_date&#x60; is greater than &#x60;date_created&#x60; * &#x60;false&#x60; - only return orders where &#x60;send_date&#x60; is equal to &#x60;date_created&#x60; 
    # @option opts [Hash<String, String>] :send_date Filter by date sent.
    # @option opts [MailType] :mail_type A string designating the mail postage type: * &#x60;usps_first_class&#x60; - (default) * &#x60;usps_standard&#x60; - a [cheaper option](https://lob.com/pricing/print-mail#compare) which is less predictable and takes longer to deliver. &#x60;usps_standard&#x60; cannot be used with &#x60;4x6&#x60; postcards or for any postcards sent outside of the United States.  (default to 'usps_first_class')
    # @option opts [SortBy4] :sort_by Sorts items by ascending or descending dates. Use either &#x60;date_created&#x60; or &#x60;send_date&#x60;, not both. 
    # @return [SelfMailerList]
    def list(opts = {})
      data, _status_code, _headers = self_mailers_list_with_http_info(opts)
      data
    end

    # list
    # Returns a list of your self_mailers. The self_mailers are returned sorted by creation date, with the most recently created self_mailers appearing first.
    # @param [Hash] opts the optional parameters
    # @option opts [Integer] :limit How many results to return. (default to 10)
    # @option opts [String] :before A reference to a list entry used for paginating to the previous set of entries. This field is pre-populated in the &#x60;previous_url&#x60; field in the return response. 
    # @option opts [String] :after A reference to a list entry used for paginating to the next set of entries. This field is pre-populated in the &#x60;next_url&#x60; field in the return response. 
    # @option opts [Array<String>] :include Request that the response include the total count by specifying &#x60;include[]&#x3D;total_count&#x60;. 
    # @option opts [Hash<String, Time>] :date_created Filter by date created.
    # @option opts [Hash<String, String>] :metadata Filter by metadata key-value pair&#x60;.
    # @option opts [Array<SelfMailerSize>] :size The self mailer sizes to be returned.
    # @option opts [Boolean] :scheduled * &#x60;true&#x60; - only return orders (past or future) where &#x60;send_date&#x60; is greater than &#x60;date_created&#x60; * &#x60;false&#x60; - only return orders where &#x60;send_date&#x60; is equal to &#x60;date_created&#x60; 
    # @option opts [Hash<String, String>] :send_date Filter by date sent.
    # @option opts [MailType] :mail_type A string designating the mail postage type: * &#x60;usps_first_class&#x60; - (default) * &#x60;usps_standard&#x60; - a [cheaper option](https://lob.com/pricing/print-mail#compare) which is less predictable and takes longer to deliver. &#x60;usps_standard&#x60; cannot be used with &#x60;4x6&#x60; postcards or for any postcards sent outside of the United States.  (default to 'usps_first_class')
    # @option opts [SortBy4] :sort_by Sorts items by ascending or descending dates. Use either &#x60;date_created&#x60; or &#x60;send_date&#x60;, not both. 
    # @return [Array<(SelfMailerList, Integer, Hash)>] SelfMailerList data, response status code and response headers
    def self_mailers_list_with_http_info(opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug 'Calling API: SelfMailersApi.self_mailers_list ...'
      end
      if @api_client.config.client_side_validation && !opts[:'limit'].nil? && opts[:'limit'] > 100
        fail ArgumentError, 'invalid value for "opts[:"limit"]" when calling SelfMailersApi.self_mailers_list, must be smaller than or equal to 100.'
      end

      if @api_client.config.client_side_validation && !opts[:'limit'].nil? && opts[:'limit'] < 1
        fail ArgumentError, 'invalid value for "opts[:"limit"]" when calling SelfMailersApi.self_mailers_list, must be greater than or equal to 1.'
      end

      if @api_client.config.client_side_validation && !opts[:'metadata'].nil? && opts[:'metadata'].to_s.length > 500
        fail ArgumentError, 'invalid value for "opts[:"metadata"]" when calling SelfMailersApi.self_mailers_list, the character length must be smaller than or equal to 500.'
      end

      pattern = Regexp.new(/[^"\\]{0,500}/)
      if @api_client.config.client_side_validation && !opts[:'metadata'].nil? && opts[:'metadata'] !~ pattern
        fail ArgumentError, "invalid value for 'opts[:\"metadata\"]' when calling SelfMailersApi.self_mailers_list, must conform to the pattern #{pattern}."
      end

      # resource path
      local_var_path = '/self_mailers'

      # query parameters
      query_params = opts[:query_params] || {}
      query_params[:'limit'] = opts[:'limit'] if !opts[:'limit'].nil?
      query_params[:'before'] = opts[:'before'] if !opts[:'before'].nil?
      query_params[:'after'] = opts[:'after'] if !opts[:'after'].nil?
      query_params[:'include'] = @api_client.build_collection_param(opts[:'include'], :multi) if !opts[:'include'].nil?
      query_params[:'date_created'] = opts[:'date_created'] if !opts[:'date_created'].nil?
      query_params[:'metadata'] = opts[:'metadata'] if !opts[:'metadata'].nil?
      query_params[:'size'] = @api_client.build_collection_param(opts[:'size'], :multi) if !opts[:'size'].nil?
      query_params[:'scheduled'] = opts[:'scheduled'] if !opts[:'scheduled'].nil?
      query_params[:'send_date'] = opts[:'send_date'] if !opts[:'send_date'].nil?
      query_params[:'mail_type'] = opts[:'mail_type'] if !opts[:'mail_type'].nil?
      query_params[:'sort_by'] = opts[:'sort_by'] if !opts[:'sort_by'].nil?

      # header parameters
      header_params = opts[:header_params] || {}
      # HTTP header 'Accept' (if needed)
      header_params['Accept'] = @api_client.select_header_accept(['application/json'])

      # form parameters
      form_params = opts[:form_params] || {}

      # http body (model)
      post_body = opts[:debug_body]

      # return_type
      return_type = opts[:debug_return_type] || 'SelfMailerList'

      # auth_names
      auth_names = opts[:debug_auth_names] || ['basicAuth']

      new_options = opts.merge(
        :operation => :"SelfMailersApi.self_mailers_list",
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => return_type
      )

      data, status_code, headers = @api_client.call_api(:GET, local_var_path, new_options)
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: SelfMailersApi#self_mailers_list\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end
  end
end
